/* crcsum.c                                                vi:se ts=8 sw=8: */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>

#include "crctab.h"
#include "pcrc.h"

int main(int argc, char *argv[])
{
	unsigned char buf[BUFSIZ];
	const char *fmt = "%lu %lu %s\n", *mode = "rb";
	int i;
	for (i = 1; i < argc; i++) {
		FILE *stream = stdin;
		size_t rd;
		off_t len = 0;
		unsigned long crc = 0xFFFFFFFF;

		if (*argv[i] == '-')
			switch (argv[i][1]) {
			case 'x':
				fmt = "%08X %lu %s\n";
				continue;
			case 'a':
				mode = "r";
				continue;
			case '\0':
				break;
			default:
				continue;
			}
		else if (!(stream = fopen(argv[i], mode))) {
			fprintf(stderr, "%s: %s: %s\n", argv[0], argv[i],
				strerror(errno));
			continue;
		}

		do
			if ((len += rd = fread(buf, 1, sizeof buf, stream)))
				crc = pcrcsum(crc, buf, rd, &pcrctab);
		while (rd == sizeof buf);

		printf(fmt, ~crc & 0xFFFFFFFF, len, argv[i]);

		if (stream != stdin)
			fclose(stream);
	}
	return EXIT_SUCCESS;
}
