/* csvaqire.c                                              vi:se ts=8 sw=8: */

#include <gdbm/ndbm.h>

#include <sys/stat.h>

#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>
#include <time.h>

#include "pcrc.h"
#include "crctab.h"
#include "csvrec.h"
#include "fcsvr.h"

int main(int argc, char *argv[])
{
	struct fcsvr r;
	int i, mode = DBM_INSERT, opmode = 0, verb = 0, ncsvs = 0, 
		ntotr = 0, nbadr = 0, nempr = 0, nerrr = 0, ndupr = 0;
	time_t start = time(NULL);
	const char *env = getenv("CSV_DATABASE"), 
	      *db_file = env ? env : "cksums";
	DBM *db;

	if ((db = dbm_open(db_file, O_RDWR | O_CREAT, DEFFILEMODE)) == NULL) {
		fprintf(stderr, "%s: %s: %s\n", argv[0], db_file,
			strerror(errno));
		return EXIT_FAILURE;
	}
	
	for (i = 1; i < argc; i++) {
		FILE *stream = stdin;
		int err = 0, j;
		char *fn = strrchr(argv[i], '/');
		if (*argv[i] == '-')
			switch (argv[i][1]) {
			case 'r':
				mode = DBM_REPLACE;
				continue;
			case 'd':
				opmode = 1;
				continue;
			case 'v':
				verb = 1;
				continue;
			case '\0':
				break;
			default:
				continue;
			}
		else if (!(stream = fopen(argv[i], "r"))) {
			fprintf(stderr, "%s: %s: %s\n", argv[0], argv[i],
				strerror(errno));
			continue;
		}
		if (!fn)
			fn = argv[i];
		else
			fn++;

		ncsvs++;

		for (j = 0; (err = getrec(stream, &r)) != -2; j++) {
			unsigned char hash[8], *p = hash + sizeof hash;
			char buf[1024];
			datum key = {hash, sizeof hash}, val = {buf, 0};
			int res = 0;

			switch (err) {
			case -1:
				fprintf(stderr, "%s: %s: #%d ignored: "
					"bad record\n", argv[0], argv[i], j);
				nbadr++;
				continue;
			case -3:
				fprintf(stderr, "%s: %s: #%d ignored: "
					"empty record\n", argv[0], argv[i], j);
				nempr++;
				continue;
			}
			
			*--p = r.size & 0xFF; r.size >>= 8;
			*--p = r.size & 0xFF; r.size >>= 8;
			*--p = r.size & 0xFF; r.size >>= 8;
			*--p = r.size & 0xFF; r.size >>= 8;

			*--p = r.crc  & 0xFF; r.crc  >>= 8;
			*--p = r.crc  & 0xFF; r.crc  >>= 8;
			*--p = r.crc  & 0xFF; r.crc  >>= 8;
			*--p = r.crc  & 0xFF; r.crc  >>= 8;

			val.dsize = 1 + snprintf(buf, sizeof buf, 
				"%s%c%s%c%s%c%s", r.fn ? r.fn : "", '\0', 
				r.desc ? r.desc : "", '\0', 
				r.path ? r.path: "", '\0', fn);
			if (val.dsize > sizeof buf)
				val.dsize = sizeof buf;

			switch (opmode) {
			case 0:
				res = dbm_store(db, key, val, mode);
				break;
			case 1:
				res = dbm_delete(db, key);
				break;
			}
			switch (res) {
			case -1:
				fprintf(stderr, "%s: %s: #%d failed: %s\n",
					argv[0], argv[i], j, 
					strerror(dbm_error(db)));
				dbm_clearerr(db);
				nerrr++;
				break;
			case 1:
				fprintf(stderr, "%s: %s: #%d collided\n",
					argv[0], argv[i], j);
				ndupr++;
				break;
			}
		}
		ntotr += j; 

		if (stream != stdin)
			fclose(stream);
	}

	dbm_close(db);
	if (verb) {
		int diff = time(NULL) - start,
			rps = diff ? ntotr * 100 / diff : 0,
			fps = diff ? ncsvs * 100 / diff : 0;
		printf("%d record%s from %d file%s in %d second%s "
			"(%d.%02d r/sec, %d.%02d f/sec)\n"
			"%d bad and %d empty records\n"
			"%d records collided and %d failed to commit\n", 
			ntotr, ntotr == 1 ? "" : "s", 
			ncsvs, ncsvs == 1 ? "" : "s", 
			diff, diff == 1 ? "" : "s",
			rps / 100, rps % 100, fps / 100, fps % 100, 
			nbadr, nempr, ndupr, nerrr);
	}
	return EXIT_SUCCESS;
}

