/* csvrec.c                                                vi:se ts=8 sw=8: */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "fcsvr.h"
#include "fn.h"

static char buf[1024] = {};

static char *csvsplit(char **src)
{
	char *result = *src, *dst = *src;
	int inner = 0;

	while (**src != '\0')
		switch (**src) {
		case '"':
			(*src)++;
			if (!inner || **src != '"') {
				inner = !inner;
				break;
			}
			/* FALLTROUGH */
		case ',':
			if (!inner)
				goto break2;
			/* FALLTROUGH */
		default:
			*dst++ = *(*src)++;
			break;
		}
break2:
	if (*src != dst)
		*dst = '\0';

	return result;
}

int getrec(FILE *stream, struct fcsvr *r)
{
	char *top, *s;

	if ((top = fgets(buf, sizeof buf, stream)) == NULL)
		return -2;

	for (;top < buf + sizeof buf; top++)
		if (*top == '\n') {
			*top = '\0';
			break;
		}
#ifdef ENABLE_CR_KLUDGE
		else if (*top == '\r' && top[1] == '\n')
			*top = '\0';
#endif
	if (top == buf + sizeof buf 
	    && (top = fgets(buf, sizeof buf, stream)) != NULL)
	{
		do
			for (;top < buf + sizeof buf; top++)
				if (*top == '\n') {
					*top = '\0';
					return -1;
				}
		while ((top = fgets(buf, sizeof buf, stream)) != NULL);
		return -1;
	}

	if (*(top = buf) == '\0')
		return -3;

	s = csvsplit(&top);
	if (*top != ',')
		return -1;
	else
		*top++ = '\0';
	r->fn = *s ? s : NULL;

	s = csvsplit(&top);
	if (*top != ',')
		return -1;
	else
		*top++ = '\0';
	r->size = strtoul(s, NULL, 10);

	s = csvsplit(&top);
	if (*top == '\0') {
		r->crc = 0;
		r->path = NULL;
	} else {
		*top++ = '\0';
		r->crc = strtoul(s, NULL, 16);

		if (*top != '"' && *top != '\\') {
			s = top;
			r->path = NULL;
		} else if ((s = csvsplit(&top), *top) != '\0') {
			*top++ = '\0';
			r->path = s;
			if (*top == '"') {
				if ((s = csvsplit(&top), *top) != '\0')
					return -1;
			} else
				s = top;
		}
	}

	r->desc = *s && strcmp(s, "unknown") && strcmp(s, "---") ? s : NULL;
	
	return 0;
}

int read_hdr(const char *csvf, int *expec, char *fmt, size_t l, 
	char *ext2, size_t l2, struct fcsvr *r)
{
	char *ext = getext(csvf);
	FILE *stream;

	if (!strcmp(ext, ".hdr"))
		return 0;
	strcpy(ext, ".hdr");

	if (!(stream = fopen(csvf, "r")))
		return 0;

	if (getrec(stream, r) != 0) {
		fclose(stream);
		return 1;
	}
	if (r->fn)
		*expec = getentc(r->fn);

	if (r->desc) {
		char *t = r->desc;
		while (*t) {
			char *m = strchr(t, '='), *n = strchr(t, ';');

			if (!m || (n && m > n))
				break;
			else if (n)
				*n++ = '\0';

			*m++ = '\0';
			
			if (!strcmp(t, "sftm"))
				strncpy(fmt, m, l - 1);
			else if (!strcmp(t, "ext"))
				strncpy(ext2, m, l2 - 1);

			t = n ? n : m + strlen(m);
		}
	}

	fclose(stream);
	return 1;
}

