/* csvwhich.c                                              vi:se ts=8 sw=8: */

#include <gdbm/ndbm.h>

#include <sys/stat.h>
#include <sys/fcntl.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>

#include "crctab.h"
#include "pcrc.h"

int main(int argc, char *argv[])
{
	unsigned char buf[BUFSIZ];
	const char *mode = "rb";
	int i, disdes = 0, list = 0;
	const char *env = getenv("CSV_DATABASE"), 
	      *db_file = env ? env : "cksums";
	DBM *db;

	if ((db = dbm_open(db_file, O_RDONLY, DEFFILEMODE)) == NULL) {
		fprintf(stderr, "%s: %s: %s\n", argv[0], db_file,
			strerror(errno));
		return EXIT_FAILURE;
	}

	for (i = 1; i < argc; i++) {
		FILE *stream = stdin;
		size_t rd;
		off_t len = 0;
		unsigned long crc = 0xFFFFFFFF;
		unsigned char hash[8], *p = hash + sizeof hash;
		datum key = {hash, sizeof hash}, val;

		if (*argv[i] == '-')
			switch (argv[i][1]) {
			case 'l':
				list = 1;
				continue;
			case 'd':
				disdes = 1;
				continue;
			case 'a':
				mode = "r";
				continue;
			case '\0':
				break;
			default:
				continue;
			}
		else if (!(stream = fopen(argv[i], mode))) {
			fprintf(stderr, "%s: %s: %s\n", argv[0], argv[i],
				strerror(errno));
			continue;
		}

		do
			if ((len += rd = fread(buf, 1, sizeof buf, stream)))
				crc = pcrcsum(crc, buf, rd, &pcrctab);
		while (rd == sizeof buf);

		crc = ~crc & 0xFFFFFFFF;

		*--p = len & 0xFF; len >>= 8;
		*--p = len & 0xFF; len >>= 8;
		*--p = len & 0xFF; len >>= 8;
		*--p = len & 0xFF; len >>= 8;

		*--p = crc & 0xFF; crc >>= 8;
		*--p = crc & 0xFF; crc >>= 8;
		*--p = crc & 0xFF; crc >>= 8;
		*--p = crc & 0xFF; crc >>= 8;

		val = dbm_fetch(db, key);

		if (val.dptr) {
			char *top = val.dptr, *d[4];
			int j;

			for (j = 0; j < 4; j++)
				if (top - val.dptr < val.dsize) {
					d[j] = top;
					top += strlen(top) + 1;
				} else
					d[j] = NULL;

			if (list)
				printf("%s %s%s%s%s\n", argv[i], 
					d[2] ? d[2] : "", d[0], 
					d[3] ? " " : "", 
					d[3] ? d[3] : "");
			else
				printf("%s\n", d[disdes]);

		} else if (!list)
			printf("%s not found\n", argv[i]);

		if (stream != stdin)
			fclose(stream);
	}
	dbm_close(db);
	return EXIT_SUCCESS;
}
